// xfail-fast

// Copyright 2012 The Rust Project Developers. See the COPYRIGHT
// file at the top-level directory of this distribution and at
// http://rust-lang.org/COPYRIGHT.
//
// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
// option. This file may not be copied, modified, or distributed
// except according to those terms.

/// Map representation

extern mod extra;

use std::io::ReaderUtil;
use std::io;
use std::str;
use std::to_str;

enum square {
    bot,
    wall,
    rock,
    lambda,
    closed_lift,
    open_lift,
    earth,
    empty
}

impl to_str::ToStr for square {
    fn to_str(&self) -> ~str {
        match *self {
          bot => { ~"R" }
          wall => { ~"#" }
          rock => { ~"*" }
          lambda => { ~"\\" }
          closed_lift => { ~"L" }
          open_lift => { ~"O" }
          earth => { ~"." }
          empty => { ~" " }
        }
    }
}

fn square_from_char(c: char) -> square {
    match c  {
      'R'  => { bot }
      '#'  => { wall }
      '*'  => { rock }
      '\\' => { lambda }
      'L'  => { closed_lift }
      'O'  => { open_lift }
      '.'  => { earth }
      ' '  => { empty }
      _ => {
        error!("invalid square: %?", c);
        fail!()
      }
    }
}

fn read_board_grid<rdr:'static + io::Reader>(in: rdr) -> ~[~[square]] {
    let in = @in as @io::Reader;
    let mut grid = ~[];
    for in.each_line |line| {
        let mut row = ~[];
        for line.iter().advance |c| {
            row.push(square_from_char(c))
        }
        grid.push(row)
    }
    let width = grid[0].len();
    for grid.iter().advance |row| { assert!(row.len() == width) }
    grid
}

mod test {
    #[test]
    pub fn trivial_to_str() {
        assert!(lambda.to_str() == "\\")
    }
}

pub fn main() {}
